#!/bin/sh
# USAGE: $0 <slot> <path> <username - optional> <password - optional>
# NOTES: use only absoulte paths (not relative)
#        <path> must NOT end with '/' (e.g. /tmp/smb/ is WRONG!)
#

# uncomment for debugging
#output /dev/console
#echo "#$0 $@"
#set -x

slot="$1"
rpath="$2"
username="$3"
password="$4"

# check if mount point exists
if [ ! -d "$rpath" ]; then
   echo "mount point does not exist: $rpath"
   exit 1
fi 

# use of "nls on" feature modifies default mshell quote processing
IFS=""

file_prefix="/tmp/smb_slot_"
touch $file_prefix$slot

# check if slot already mounted with the requested path
prev_path=`cat $file_prefix$slot`
path_tag=`echo "$rpath"|sed 's| |\\\\\\\\040|g'`
prev_path_tag=`echo $prev_path|sed 's| |\\\\\\\\040|g'`
if [ "$prev_path" = "$rpath" ]; then
   if grep "$path_tag" /proc/mounts; then
      echo "slot already mounted with the requested path"
      exit 0
   fi
fi

# number of slots using previous mount
mount_count=`cat $file_prefix*|grep '^'$prev_path'$'|wc -l|xargs printf %d`

# unmount previous path, if no other slot is using it
if [ -n "$prev_path" -a "$mount_count" = "1" ]; then
   if grep "$prev_path_tag" /proc/mounts; then
      if umount "$prev_path"; then
         sed -e 's|^'$prev_path'$||g' -e '/^$/ { d }' -i /tmp/smb_umount
      else
         grep '^'$prev_path'$' /tmp/smb_umount ||
         echo "$prev_path" >> /tmp/smb_umount
      fi
   fi
fi

# process unmounted shares
touch /tmp/smb_umount
i=`cat /tmp/smb_umount|wc -l|xargs printf %d`
while [ -n "$i" -a $i -gt 0 ]; do
  share=`tail -n$i /tmp/smb_umount|head -n1`
  used=`cat $file_prefix*|grep '^'$share'$'|wc -l|xargs printf %d`
  if [ "$used" = "0" -a "$share" != "$prev_path" ]; then
     umount "$share" &&
     sed -e 's|^'$share'$||g' -e '/^$/ { d }' -i /tmp/smb_umount
  fi
  i=`expr $i \- 1`
done

# check if already mounted
if grep "$path_tag" /proc/mounts; then
   echo "Directory already mounted..."
   echo "$rpath" > $file_prefix$slot
   exit 0
fi

# NLS stuff
. /sbin/nls.sh
srv_path=`echo $rpath|sed "s|$DEVICE_PATH/smb/||g"`
workgroup=`echo $srv_path|cut -d '/' -f 1`
hostname=`echo $srv_path|cut -d '/' -f 2`
share=`echo $srv_path|cut -d '/' -f 3`
fostype="$DEVICE_PATH/smb/$workgroup/$hostname/@os_type@"

ishostip=`echo "$hostname"|sed -r "s/[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/@ipaddr@/g"`
if [ "$ishostip" = "@ipaddr@" ]; then
   ip_addr=$hostname
else
   ifname=`cat /tmp/ifname`
   iproot=`ifconfig $ifname|grep "Bcast:*"|sed 's/.*Bcast://g'|cut -d " " -f 1`
   iproot=`echo $iproot|sed 's/\.255$//g'|sed 's/\.255$//g'|sed 's/\.255$//g'`
   ip_addr=`nmblookup $hostname|grep "^$iproot"|head -n 1|cut -d ' ' -f 1`
fi

# check to see that ip address is available
if [ -z "$ip_addr" ]; then
   echo "failed to lookup hostname $hostname"
   exit 1
fi
 
# mount with proper NLS options
smbdriver="cifs"
if [ "$smbdriver" = "cifs" ]; then
   mount.cifs "//$ip_addr/$share" "$rpath" -o ro,user=$username%$password,iocharset=$iocharset,rsize=8192,wsize=8192 &&
   echo "$rpath" > $file_prefix$slot
   exit 0
else
   if [ "$unicode" = "1" -a "$codepage" = "unicode" -a -f "$fostype" ]; then
      if grep -i "^Unix" "$fostype" ; then
         unicode=0
         codepage=utf8
      fi
   fi
   opt="lfs"
   [ "$unicode" = "1" ] && opt="lfs,unicode"
   [ "$codepage" != "utf8" -a "$codepage" != "unicode" ] && codepage=cp$codepage
   rm -f /tmp/smb_status_$slot
   smbmount "//$ip_addr/$share" "$rpath" -n -o ro,$opt,iocharset=$iocharset,codepage=$codepage,username=$username,password=$password,workgroup=$workgroup 2> /tmp/smb_status_$slot &

   # wait for mount status
   timeout=5000
   while [ "$timeout" -gt 0 ] ; do
      usleep 250000
      timeout=`expr $timeout \- 250`
      if [ -f /tmp/smb_status_$slot ]; then
         grep "failed" /tmp/smb_status_$slot && exit 1
         grep "killed" /tmp/smb_status_$slot && exit 1
      fi
      grep "$path_tag" /proc/mounts && timeout=0
   done
fi

# store new path in slot
echo "$rpath" > $file_prefix$slot
